#include "Widget.h"

#include "Defs.h"
#include "PanelPage.h"

#include <Wt/WContainerWidget>
#include <Wt/WCssDecorationStyle>
#include <Wt/WApplication>
#include <Wt/WBorder>
#include <Wt/WEvent>
#include <Wt/WPanel>
#include <Wt/WPopupMenu>
#include <Wt/WPopupMenuItem>
#include <Wt/WDialog>
#include <Wt/WText>
#include <Wt/WLink>
#include <Wt/WPushButton>
#include <Wt/WMessageBox>
#include <Wt/WInPlaceEdit>
#include <Wt/WDoubleValidator>
#include <Wt/WLineEdit>
#include <Wt/WCheckBox>
#include <Wt/WSortFilterProxyModel>
#include <Wt/WStandardItem>
#include <Wt/WStandardItemModel>

#include <iostream>
#include <string>
#include <vector>
#include <set>
#include <boost/signals.hpp>
#include <boost/bind.hpp>
#include <boost/foreach.hpp>
#include <boost/lexical_cast.hpp>

using namespace Wt;
using namespace std;
using namespace Defs;

bool Widget::mouseWentDown_(false);
bool Widget::doubleClicked_(false);

Widget::Widget(PanelPage* parent) :
	Container(),
	parent_(parent),
	selected_(false),
	border_(WBorder::Hidden, 1, WColor(100, 100, 100)),
	popup_(0)
{
	resize(100, 100);
	setAttributeValue("oncontextmenu", "event.cancelBubble = true; event.returnValue = false; return false;");
	decorationStyle().setBorder(border_);
	setPositionScheme(Absolute);
	this->mouseWentDown().connect(this, &Widget::onMouseDown);
	this->mouseWentUp().connect(this, &Widget::onMouseUp);
	this->doubleClicked().connect(this, &Widget::onDoubleClick);
}

Widget::~Widget(){
//    delete popup_;
}

void Widget::onDoubleClick(const Wt::WMouseEvent &e) {
    cout << endl<< MSGAT << "double click!!" << endl <<endl;
	doubleClicked_ = true;
}

void Widget::onMouseDown(const Wt::WMouseEvent &e) {
    cout << endl<< MSGAT << "onMouseDown!!" << endl <<endl;
	mouseWentDown_ = true;
}

void Widget::onMouseUp(const Wt::WMouseEvent &e) {
    cout << endl<< MSGAT << "onMouseUp!!" << endl <<endl;
	mouseWentDown_ = false;
    (selected_) ? select(false) : select();
    showPopup(e);
}

void Widget::select(bool b) {
	if (not selected_ && b) {
		border_.setStyle(WBorder::Dashed);
		decorationStyle().setBorder(border_);
		setOffsets(offset(Left).value() - border_.explicitWidth().value(),
				Left);
		setOffsets(offset(Top).value() - border_.explicitWidth().value(), Top);
	} else if (selected_ && not b) {
		border_.setStyle(WBorder::Hidden);
		decorationStyle().setBorder(border_);
		setOffsets(offset(Left).value() + border_.explicitWidth().value(),
				Left);
		setOffsets(offset(Top).value() + border_.explicitWidth().value(), Top);
	}
	selected_ = b;
	if (b) {
        parent_->selected().insert(this);
    } else {
        parent_->selected().erase(this);
    }
    cout << MSGAT << " selected widgets = " << parent_->selected().size() << endl;
}

std::vector<std::vector<Wt::WWidget*> > Widget::getProperties(Defs::PropertiesGroup propGr) {
	std::vector<std::vector<Wt::WWidget*> > vvw; // vector of vector of widgets
	return vvw;
}

void Widget::showPopup(const Wt::WMouseEvent& e){
    if (!popup_) {
        popup_=(new WPopupMenu());
        popup_->setAttributeValue("oncontextmenu","event.cancelBubble = true; event.returnValue = false; return false;");
        popup_->addItem("delete");
        popup_->aboutToHide().connect(this, &Widget::popupAction);
    }

    if (popup_->isHidden()) {
        popup_->refresh();
        popup_->popup(e);
    } else {
        popup_->hide();
    }
}

void Widget::popupAction() {
    mouseWentDown_ = false;
    if (popup_->result() == 0) { // click outside the popup menu
        return;
    }
    else if (popup_->result()->text() == WString("delete")) {
        WMessageBox mb;
        mb.setModal(true);
        mb.setCaption("confirm");
        mb.setText("are you sure you want to delete the widget");
        mb.setButtons(Yes | No);
        mb.button(Yes)->setText("yes");
        mb.button(Yes)->clicked().connect(&mb, &Wt::WDialog::accept);
        mb.button(No)->setText("no");
        mb.button(No)->clicked().connect(&mb, &Wt::WDialog::reject);
        mb.button(No)->setFocus();
        mb.rejectWhenEscapePressed();
        if (mb.exec() == WDialog::Accepted) {
            parent_->selected().erase(this);
            cout << "delete" << endl;
            delete this;
            cout << "now return" << endl;
            return;
        }
    }
    else {
        assert(false);
    }
}

